%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% EFFICIENT and INCENTIVE COMPATIBLE LIVER EXCHANGE
% Econometrica
% (c) Ergin, Sonmez, Unver, 2017 version 1
% this version uses CPLEX of IBM for integer programming
%% k-way weighted maximization function for single-graft 
%% liver exchange function for MATLAB
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [Mu, Matched_true, matchnum]=...
    WeightMax_K_WAY_one_lobe_liver_exchange(n, Cyc_size, Cycles, In_cyc, W_cyc, num_cyc_size, Same_pat, ...
    weight_true, cplex_true);
Mu=zeros(n,1);
Matched_true=zeros(n,1);
matchnum=0;

maxw_x=[];
Cycles_x=[];
In_x = Cyc_size <= num_cyc_size;
Cycles_x=Cycles(In_x,:);
Cyc_size_x=Cyc_size(In_x);
In_cyc_x=In_cyc(In_x,:);
W_cyc_x=W_cyc(In_x);
for ss=1:size(Same_pat,1);
    pat1=Same_pat(ss,1);
    pat2=Same_pat(ss,2);
    a1=pat1==Cycles_x(:,1);
    a2=pat1==Cycles_x(:,2);
    a3=pat1==Cycles_x(:,3);
    a4=pat1==Cycles_x(:,4);
    b1=pat2==Cycles_x(:,1);
    b2=pat2==Cycles_x(:,2);
    b3=pat2==Cycles_x(:,3);
    b4=pat2==Cycles_x(:,4);
    ac=a1|a2|a3|a4;
    bc=b1|b2|b3|b4;
    ii=~(ac&bc);
    Cycles_x=Cycles_x(ii>0,:);
    In_cyc_x=In_cyc_x(ii>0,:);
    W_cyc_x=W_cyc_x(ii>0,:);
    Cyc_size_x=Cyc_size_x(ii>0,:);
end;
num_x=size(In_cyc_x,1);
xU=ones(num_x,1);
xL=zeros(num_x,1);
f=-Cyc_size_x;
if weight_true
    w=-W_cyc_x;
else
    w=f;
end;
A=In_cyc_x';
bU=ones(n,1);
bL=zeros(n,1);
for ss=1:size(Same_pat,1);
    pat1=Same_pat(ss,1);
    pat2=Same_pat(ss,2);
    a1=pat1==Cycles_x(:,1);
    a2=pat1==Cycles_x(:,2);
    a3=pat1==Cycles_x(:,3);
    a4=pat1==Cycles_x(:,4);
    b1=pat2==Cycles_x(:,1);
    b2=pat2==Cycles_x(:,2);
    b3=pat2==Cycles_x(:,3);
    b4=pat2==Cycles_x(:,4);
    ac=a1|a2|a3|a4;
    bc=b1|b2|b3|b4;
    ccs=(ac|bc);
    A=[A;ccs'];
    bL=[bL;0];
    bU=[bU;1];
end;

if num_x>0
    if ~cplex_true
        an=size(A,1);
        CTYPE=zeros(an,1);
        CTYPE(1:an,1)='U';
        
        CTYPE=char(CTYPE);
        
        VARTYPES=zeros(num_x,1);
        VARTYPES(1:num_x,1)='I';
        VARTYPES=char(VARTYPES);
        [X1,FOPT,LPSOLVED,STATUS]=glpk(w,A,bU,xL,xU,CTYPE,VARTYPES,1);
    else
        VARTYPES2=zeros(1,num_x);
        VARTYPES2(1,1:num_x)='B';
        
        VARTYPES2=char(VARTYPES2);
        [X1,FOPT,LPSOLVED,STATUS]= cplexmilp(w,A,bU,[],[],[],[],[],xL,xU,VARTYPES2);
    end;
    
    X2=round(X1);
    fmin=f'*X2;
    matchnum=-fmin;
    [Ch,Y]=find(X2);
    num_cy=size(Ch,1);
    for cc=1:num_cy
        cyc=Ch(cc);
        for ii=1:Cyc_size_x(cyc)
            pat=Cycles_x(cyc,ii);
            jj=(ii+1)*(Cyc_size_x(cyc)>ii)+1*(Cyc_size_x(cyc)==ii);
            don=Cycles_x(cyc,jj);
            Matched_true(pat,1)=1;
            Mu(pat,1)=don;
        end;      
    end;
end;