%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% EFFICIENT and INCENTIVE COMPATIBLE LIVER EXCHANGE
% Econometrica
% (c) Ergin, Sonmez, Unver, 2017 version 1
%`version 3 - revision May 2019
% this version uses CPLEX of IBM for integer programming
%% Patient Generator Function for MATLAB
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [PList, C_L, C_R, ABOc, Cat_num, Topo_c456]=...
    createpatientDICH_LIVER_exchange_KOREA_3rev(n, ABOdistp, ABOdistd, femp_ratio, femd_ratio, ...
    Weight_dist, Height_dist, Leftlobe_dist, w_perc, volreq_perc);

leftlobe=0.35;
ABOc=zeros(n,n);    %ABOC(i,j)=1 menas that patient i and donor j are ABOComple
C_L=zeros(n,n);     %C_L(i,j)=1 means that patient i and donor j are ABOComple and  negative crossmatch
C_R=zeros(n,n);

%totalgen=0;        %how many trials were needed to generate the n pair list
PList=zeros(n,21);
Cat_num=zeros(6,1); %see categories at the earlier version https://ideas.repec.org/p/boc/bocoec/951.html

for i=1:n
    %totalgen=totalgen+1;
    New_rand=rand(5,1);
    Bloodt=zeros(1,2);
    Graft_vo=zeros(1,2);
    Female=zeros(1,2);
    for kk=1:2
        z=1;
        if kk==1
            ABOdist=ABOdistp;
            Female(kk)=(New_rand(2+kk)<femp_ratio);
        else
            ABOdist=ABOdistd;
            Female(kk)=(New_rand(2+kk)<femd_ratio);
        end;
        while New_rand(kk)>ABOdist(z)
            z=z+1;
        end;
        Bloodt(kk)=z;
        
        if Female(kk)
            %Wes=Weight_dist(:,2);
            Hes=Height_dist(:,2);
        else
            %Wes=Weight_dist(:,1);
            Hes=Height_dist(:,1);
        end;
        
        %weight=trunrndn([1 1],Wes(1),Wes(2),Wes(3),Wes(4));
        height=trunrndn([1 1],Hes(1),Hes(2),Hes(3),Hes(4));
        
        if Female(kk)
            weight=exp(3.49+1.45*log(height));
        else
            weight=exp(3.28+1.92*log(height));
        end;
        BSA=(weight*height*100/3600)^(.5);
        livvol=(893.485*BSA-439.169)/1000;
        if kk>1
            %leftlobe=trunrndn([1 1],Leftlobe_dist(1),Leftlobe_dist(2),Leftlobe_dist(3),Leftlobe_dist(4));
            Graft_vo(kk)=leftlobe*livvol;
        else
            Graft_vo(kk)=volreq_perc*livvol;
        end;
        
    end;
    
    for kk=1:2
        b=Bloodt(kk);
        if b==1
            bb=[1 1];
        elseif b==2
            bb=[0 1];
        elseif b==3
            bb=[1 0];
        else
            bb=[0 0];
        end;
        if kk==1
            Xb=bb;
        else
            Yb=bb;
        end;
    end;
    
    right_graft_vo=(Graft_vo(2)/leftlobe)*(1-leftlobe);
    X=[Xb Graft_vo(1)]';
    Y=[Yb Graft_vo(2)]';
    rhoY=[Yb right_graft_vo]';
    cat=0;
    if isgr(X,rhoY)
        cat=1;
    elseif isgre(Y,X)
        cat=2;
    elseif isgr(X,Y)&isgre(rhoY,X)
        cat=3;
    elseif isgr(X,Y)&(~isgre(X,rhoY))&(~isgre(rhoY,X))
        cat=4;
    elseif (~isgre(rhoY,X))&(~isgre(X,Y))
        cat=5;
    elseif isgr(rhoY,X)&(~isgre(X,Y))&(~isgre(Y,X))
        cat=6;
    end;
    Cat_num(cat)=Cat_num(cat)+1;
    w_true = New_rand(5,1) < w_perc;
    priority = i;
    PList(i,1:19)=[Bloodt Graft_vo right_graft_vo Female w_true cat X' Y' rhoY' priority];
    %1x2 blood types, liv req, donor-left lobe, donor right-lobe, 1x2 Female1by2 or not, 1x1 willing, pair category, 1x3 patient type, 1x3 donor type, 1x3 donor rho type
end;
Prec_c456=zeros(n,n);
Sc_Ltrue=zeros(n,1);
Sc_Rtrue=zeros(n,1);

for i=1:n
    Xi=(PList(i,10:12))'; %patient type
    Yi=(PList(i,13:15))'; %donor type with L-Lobe donation
    rhoYi=(PList(i,16:18))'; %donor type with R-Lobe donation
    if ~isgr(rhoYi,Yi)
        [rhoYi Yi]
        'error'
        pause
    end;
    Sc_Ltrue(i)=isgre(Yi,Xi);
    Sc_Rtrue(i)=isgre(rhoYi,Xi);
    C_L(i,i)=Sc_Ltrue(i);
    C_R(i,i)=Sc_Rtrue(i);
    for j=i+1:n
        Xj=(PList(j,10:12))'; %patient type
        Yj=(PList(j,13:15))'; %donor type with L-Lobe donation
        rhoYj=(PList(j,16:18))'; %donor type with R-Lobe donation
        if ~isgr(rhoYj,Yj)
            [rhoYj Yj]
            'error'
            pause
        end;       
        if sum(([Xi;Yi]==[Xj;Yj]))==6           
            [[Xi;Yi] [Xj;Yj]]            
            pause
        end;
        C_L(i,j)=isgre(Yj,Xi);
        C_R(i,j)=isgre(rhoYj,Xi);
        C_L(j,i)=isgre(Yi,Xj);
        C_R(j,i)=isgre(rhoYi,Xj);
        if (PList(i,9)>3) & (PList(j,9)>3)
            if isgre(Yj,Xi) & (~isgre(Yi,Xj)) & isgre(rhoYi,Xj)
                Prec_c456(i,j)=1;
            elseif isgre(Yi,Xj) & (~isgre(Yj,Xi)) & isgre(rhoYj,Xi)
                Prec_c456(j,i)=1;
            end;
        end;
    end;
end;
Pats_c456 = PList(:,9)>3;
[Id_c456,Y]=find(Pats_c456);

R_Prec_c456 = sparse(Prec_c456(Pats_c456,Pats_c456'));
R_Topo_c456 = graphtopoorder(R_Prec_c456);
Topo_c456=Id_c456(R_Topo_c456);
%[Topo_c456 PList(Topo_c456,10:18)]
%view(biograph(R_Prec_c456))
%R_Topo_c456
%pause;
PList(:,20:21)=[Sc_Ltrue Sc_Rtrue];

