%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% EFFICIENT and INCENTIVE COMPATIBLE LIVER EXCHANGE
% Econometrica
% (c) Ergin, Sonmez, Unver, 2017 version 1
%`version 3 - revision May 2019
% this version uses CPLEX of IBM for integer programming%
%% A Greedy 2-way Liver Exchange Function for MATLAB
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [Mu, matchnumL, matchnumR, totalnum] = ...
    greedy_ref_2way_liverexchange_3rev (C_L, C_R, Will, Cats);

n=size(C_L,1);
Pats_c5 = (Cats==5);
n_c5 = sum(Pats_c5);

Pats_c2 = (Cats==2); 
Pats_c3_w = Will & (Cats==3); %Cat 3 and willing pairs
Sc_step1 = Pats_c2 | Pats_c3_w;

Pats_c4_w = Will & (Cats==4);
Pats_c5_w = Will & (Cats==5);

Pats_c6_w = Will & (Cats==6);
n_c6_w=sum(Pats_c6_w);

selfnumL_c2 = sum(Pats_c2);
selfnumR_c3 = sum(Pats_c3_w);
selfnumR_c6 = sum(Pats_c6_w);

Pats_in = ~Sc_step1;

%if ~isequal(Pats_c4_w .* Sc_c3,zeros(n,1));
%    'Error'
%    'Cat 2 and L Compatible Pairs are not the same'
%    pause
%end;

CompL = C_L;

CompL(Sc_step1,:) = 0;
CompL(:,Sc_step1') = 0;

CompR = C_L;
Right_transform = Pats_c4_w | Pats_c5_w;
CompR(:,Right_transform') = C_R(:,Right_transform');
CompR(Sc_step1,:) = 0;
CompR(:,Sc_step1') = 0;

E_L=CompL.*CompL';
E_R=(~E_L).*(CompR.*CompR');
[I_1_L,I_2_L]=find(E_L);
[I_1_R,I_2_R]=find(E_R);

nL=size(I_1_L,1);
nR=size(I_1_R,1);

NL=randperm(nL);
NR=randperm(nR);

NX=[NL NR]';

Mu=zeros(n,1);

for ii=1:nL;
  edge=NL(ii);
  i1=I_1_L(edge);
  i2=I_2_L(edge);
  if (Mu(i1)==0)&(Mu(i2)==0)
    Mu(i1)=i2;
    Mu(i2)=i1;
  end;
end;

for ii=1:nR
  edge=NR(ii);
  i1=I_1_R(edge);
  i2=I_2_R(edge);
  if (Mu(i1)==0)&(Mu(i2)==0)
    Mu(i1)=i2;
    Mu(i2)=i1;
  end;
end;

Matched_true=1*(Mu>0);

R_matched_true=zeros(n,1);

for i=1:n
    if Matched_true(i)
      R_matched_true(i) = (C_L(Mu(i),i)==0);
    elseif Pats_c2(i) 
      Mu(i)=i;
      Matched_true(i)=1;
    elseif Pats_c3_w(i)||Pats_c6_w(i)    
      Mu(i)=i;
      Matched_true(i)=1;
      R_matched_true(i)=1;
    end;
end;
totalnum = sum(Matched_true);
matchnumR = sum(R_matched_true);
matchnumL = totalnum - matchnumR;