%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% EFFICIENT and INCENTIVE COMPATIBLE LIVER EXCHANGE
% INCENTIVE COMPATIBLE AND EFFICIENT LIVER EXCHANGE
% Econometrica
% (c) Ergin, Sonmez, Unver, 2017 version 1
%`version 3 - revision May 2019
% this version uses CPLEX of IBM for integer programming%
%% IC+PE+IR 2-way Liver Exchange Mechanism with dich. receiving  prefs
%% for MATLAB (also MAX-IR matching can be calculated using this)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [Mu, matchnumL, matchnumR, totalnum] = ...
    ic_2way_liverexchange_3rev (C_L, C_R, Will, Cats, Topo_c456, Prvalues, ...
    cplex_true,  ic_true);

n=size(C_L,1);

Pats_c456 = (Cats>3);
n_c456 = sum(Pats_c456);

Pats_c2 = (Cats==2);
Pats_c3_w = Will & (Cats==3); %Cat 3 and willing pairs
Sc_step1 = Pats_c2 | Pats_c3_w;

Pats_c4_w = Will & (Cats==4);
Pats_c456_w = Will & (Cats>3);

Pats_c6_w = Will & (Cats==6);
n_c6_w=sum(Pats_c6_w);

selfnumL_c2 = sum(Pats_c2);
selfnumR_c3 = sum(Pats_c3_w);
selfnumR_c6 = sum(Pats_c6_w);

Mu=zeros(n,1);



Comp = C_L;

Comp(Sc_step1,:) = 0;
Comp(:,Sc_step1') = 0;

Pr_rank_in = zeros(n,1);
Pr_rank_in(1:n_c456,1) = Topo_c456;
%if ic_true
Prvalues_notc456 = Prvalues .* (~Pats_c456);
[Y, Pr_rank_notc456] = sort(Prvalues_notc456, 1, 'descend');
Pr_rank_in(n_c456+1:n,1) = Pr_rank_notc456(1:n-n_c456,1);


num_cyc_size = 2;
Same_pat = [];
weight_true = 1;

R_match_true=zeros(n,1);
Prvalues_in = zeros(n,1);

for r = 1:n
  pat = Pr_rank_in(r);
  Prvalues_in(pat) = n-r+1;
end;
if ~ic_true
  Prvalues_in=Prvalues;
end;
  

Prvalues_in = Prvalues_in .* (~Sc_step1);

if ic_true
    %Comp(:,Pats_c4_w') = C_R(:,Pats_c4_w');
    %Comp(Sc_step1,Pats_c4_w')=0;
    
    n_c456_reached_true = false;
    Right_transform = zeros(n,1);
    %Pats_in = zeros(n,1);
    Pats_in=(~Sc_step1);
    r = 1;
    pat = 0;
    while (r <= n_c456 + 1) && (~n_c456_reached_true)
        if r == n_c456 + 1
            n_c456_reached_true=true;
            %Pats_in=(~Sc_step1);
        else
            pat = Pr_rank_in(r,1);
            Pats_in(pat,1) = 1;
        end;
        if n_c456_reached_true | Will(pat)
            %if (pat>0) & Sc(pat)
            %    'Error'
            %    pause
            %end;
            C=Comp;
            C(Pats_in==0,(Pats_in==0)')=0;
            C2way = C' .* C;
            %[h,k,l]=find(C2way)
            %pause
            %C2way(Pats_in==0, (Pats_in==0)') = 0;
            W2way = zeros(n, n);
            for i=1:n
                W2way(:,i) = Prvalues_in(i) * C2way(:,i) + W2way(:,i);
                W2way(i,:) = Prvalues_in(i) * C2way(i,:) + W2way(i,:);
                W2way(i,1:i) = 0;
                C2way(i,1:i) = 0;
            end;
            %if ~ic_true
            %[I1, I2, W_cyc] = find(W2way)
            %sparse(C2way)
            %sparse(W2way)
            %pause
            %else
            [I1, I2, W_cyc] = find(W2way);
            [J1, J2, C_cyc] = find(C2way);
           % pause;
            if ~isequal(J1,I1) | ~isequal(I2,J2)
                'error'
                
                [I1 I2 W_cyc]
                [J1 J2 C_cyc]
            pause
            end;
            
            %pause;
            %end;
            Cycles=[I1 I2];
            num_cyc = size(Cycles, 1);
            Cyc_size = 2*ones(num_cyc, 1);
            In_cyc = zeros(num_cyc, n);
            for cyc = 1:num_cyc
                In_cyc(cyc, I1(cyc)) = 1;
                In_cyc(cyc, I2(cyc)) = 1;
            end;
            [Mu, Matched, totalnum]=...
                WeightMax_K_WAY_one_lobe_liver_exchange(n, Cyc_size, Cycles, In_cyc, W_cyc, num_cyc_size, ...
                Same_pat, weight_true, cplex_true);
            %[Mu_t, Matched_true_t, matchnum_t]=...
            %    WeightMax_K_WAY_one_lobe_liver_exchange(n, Cyc_size, Cycles, In_cyc, W_cyc, num_cyc_size, ...
            %    Same_pat, false, cplex_true);
            %if matchnum~=matchnum_t
            %    'Error'
            %    matchnum
            %    matchnum_t
            %    pause
            %end;
            if (~n_c456_reached_true) && (~Matched(pat)) && Will(pat) && (~Pats_c6_w(pat))          
                Comp(:,pat)=C_R(:,pat);
                Comp(Sc_step1,pat)=0;
                Right_transform(pat)=1;
                %pat
                %find(Right_transform)
                %find(Matched)
                %pause
            end;
        end;
        r=r+1;
    end;
else
    Pats_in = ~Sc_step1;
    Comp(~Pats_in,~Pats_in')=0;
    
    Right_transform = (Pats_c456_w) & (~Pats_c6_w) ;
    Comp(:,Right_transform') = C_R(:,Right_transform');
    Comp(~Pats_in,Right_transform')=0;
    
    C=Comp;
    %C(Pats_in==0,(Pats_in==0)')=0;
    C2way = C' .* C;
    W2way = zeros(n, n);
    for i=1:n
        W2way(:,i) = Prvalues_in(i) * C2way(:,i) + W2way(:,i);
        W2way(i,:) = Prvalues_in(i) * C2way(i,:) + W2way(i,:);
        W2way(i,1:i) = 0;
        C2way(i,1:i) = 0;        
        if Pats_c6_w(i)
            C2way(i,i)=1;
            W2way(i,i)=Prvalues_in(i);
        end;
    end;

    
    %if ~ic_true
    %[I1, I2, W_cyc] = find(W2way)
    %sparse(C2way)
    %sparse(W2way)
    %pause
    %else
    [I1, I2, W_cyc] = find(W2way);
    [J1, J2, C_cyc] = find(C2way);
    
    if ~isequal(J1,I1) | ~isequal(I2,J2)
        'error' 
        
        [I1 I2 W_cyc]
        [J1 J2 C_cyc]
        pause
    end;
    
    %pause;
    %end;
    Cycles=[I1 I2];
    Cycles(I1==I2,2)=0;
    num_cyc = size(Cycles, 1);
    Cyc_size = sum(Cycles>0,2);
    In_cyc = zeros(num_cyc, n);
    for cyc = 1:num_cyc
        In_cyc(cyc, I1(cyc)) = 1;
        In_cyc(cyc, I2(cyc)) = 1;
    end;
    
    [Mu, Matched, totalnum]=...
        WeightMax_K_WAY_one_lobe_liver_exchange(n, Cyc_size, Cycles, In_cyc, W_cyc, num_cyc_size, ...
        Same_pat, weight_true, cplex_true);
    %[Mu_t, Matched_true_t, matchnum_t]=...
    %            WeightMax_K_WAY_one_lobe_liver_exchange(n, Cyc_size, Cycles, In_cyc, W_cyc, num_cyc_size, ...
    %            Same_pat, false, cplex_true);
    %        if matchnum~=matchnum_t
    %            'Error'
    %            matchnum
    %            matchnum_t
    %            pause
    %            end;
end;
R_match_true=zeros(n,1);
leftleft=0; %notused
for i=1:n
    if (Matched(i)>0)
      j=Mu(i);
      R_match_true(i) = (C_L(j,i)==0); 
      if ~R_match_true(i)&&(C_L(i,j)==1)
        leftleft=leftleft+1; %notused
      end;
    end;
    if ~R_match_true(i)
        if Pats_c6_w(i) && (Matched(i)==0)
            R_match_true(i)=1;
            Mu(i)=i;
            totalnum=totalnum+1;
        elseif Pats_c2(i)
            Mu(i)=i;
            totalnum=totalnum+1;
        elseif Pats_c3_w(i)
            R_match_true(i)=1;
            Mu(i)=i;
            totalnum=totalnum+1;
        end
    end;
end;


matchnumR = sum(R_match_true);
matchnumL = totalnum - matchnumR;
