%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% EFFICIENT and INCENTIVE COMPATIBLE LIVER EXCHANGE
% Econometrica
% (c) Ergin, Sonmez, Unver, 2017 version 1
%`version 4 - revision May 2019
% this version uses CPLEX of IBM for integer programming
%% Main Simulation File for MATLAB
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

diary off;
clear
fnamestr='eff-ic-liver-KOREA-4rev-';

%the category language and numbers used below follow the first draft of the paper, 
%please refer to it at https://ideas.repec.org/p/boc/bocoec/951.html

%number of simulation to average over
max_sim=1000;

%patient population sizes
Pops(1)=50;
Pops(2)=100;
Pops(3)=250;
ini_pop=1;
max_pop=3;

%risks of death from donation / not used in the paper
right_risk=0.005;
left_risk=0.001;

%willingness rates considered
W_rate(1)=0;
W_rate(2)=.2;
W_rate(3)=.4;
W_rate(4)=.6;
W_rate(5)=.8;
W_rate(6)=1;

ini_W_rate=1;
max_W_rate=6;
%whole population standard error is reported not the average's standard
%error
denomstd=1;
%uses "cplex" optimization or not
cplex_true=true;

set(0,'RecursionLimit',1000);
max_size_rate=1; %fix at 1 

dstr=date;
tstr=strcat('Rnd',int2str(round(rand*1000)));
str1=strcat('Pop',int2str(max_pop));

str2=strcat('W',int2str(max_W_rate));

dname=strcat(fnamestr,'OUTPUT-',dstr,'-',tstr,'-',str1,'-',str2,'.txt');
figname=strcat(fnamestr,'OUTPUT-',dstr,'-',tstr,'-',str1,'-',str2,'.fig');
figname2=strcat(fnamestr,'Referee-',dstr,'-',tstr,'-',str1,'-',str2,'.fig');

size_comp_true=1;
diary(dname);

cp=3; % for patient variation of agent sizes - 3 standard deviations truncated normal
cd=3; % for donor

%korean data statistics retrieve
ABOdistp=csvread('ABOdist_KOREA.csv')

ABOdistd=csvread('ABOdist_KOREA.csv')
Weight_dist=csvread('weightdist_KOREA.csv') %first column male, second column female: rows mean, std, min, max
Height_dist=csvread('heightdist_KOREA.csv') %first column male, second column female: rows mean, std, min, max
Leftlobe_dist=csvread('donorleftlobedist_KOREA.csv') %percentage of total volume rows mean, std, min, max
%female ratio in patients and donors in Korea
femp_ratio=0.346
femd_ratio=0.266

size_rate=0.4;


Ansavrstd=[];


for num_pop=ini_pop:max_pop
  Res_temp_w0=[];
  
  for num_size_rate=1:max_size_rate
    for num_W_rate=ini_W_rate:max_W_rate
      Res_temp=[];
      w_perc=W_rate(num_W_rate);
      
      volreq_perc=size_rate(num_size_rate);
      seed=10*9*8*7*6*5*4*3*2;
      n=Pops(num_pop);
      diary on;
      fprintf('Running for population size %i; Running for size threshold: %4.1f \n', n, volreq_perc);
      diary off;
      Stats=[];
      for num_sim=1:max_sim
        %fprintf('Pop: %4i; Sim: %4i; ', n, num_sim);
        seed0=seed*num_sim+1;
        S=rng;
        rng(seed0,'twister');
        tic;
        %patient-donor pool is generated
        [PList, C_L, C_R, ABOC, Cat_num, Topo_c456]=...
          createpatientDICH_LIVER_exchange_KOREA_3rev(n, ABOdistp, ABOdistd, femp_ratio, femd_ratio, ...
          Weight_dist, Height_dist, Leftlobe_dist, w_perc, volreq_perc);
        rng(S);
        
        
        t=toc;
        tic;
        %fprintf('Pop generated, time: %8.2f; ',t);
        
        Cats=PList(:,9); %Categories of Pairs
        Cat456=(Cats>3); %category 4,5,6 pairs
        cat456num=sum(Cat456);
        Will=PList(:,8); %Willingness of Pairs
        Self_L=PList(:,20);
        
        Self_R_w=PList(:,21)&Will&(~Self_L);
        
        Prvalues=PList(:,19); %Priority Values of Pairs
        
        %our IC+IR+PE adaptive-priorty  matching
        ic_true=true;
        [Mu_IC,matchnumL_IC, matchnumR_IC, totalnum_IC] = ...
          ic_2way_liverexchange_3rev(C_L, C_R, Will, Cats, Topo_c456, Prvalues, cplex_true, ic_true);
        t=toc;
        tic;
       
        %RSU priority matching, baseline
        n_RSU=size(C_L,1);
        C_R_RSU=C_L;
        for i=1:n
          C_R_RSU(i,i)=C_R(i,i);
        end;
        ic_true=false;
        [Mu_RSU,matchnumL_RSU, matchnumR_RSU, totalnum_RSU] = ...
          ic_2way_liverexchange_3rev(C_L, C_R_RSU, Will, Cats, Topo_c456, Prvalues, cplex_true, ic_true);
        t=toc;
        tic;
        
        %fprintf('IC Match exchange, time: %8.2f; ',t);
       
        %maximal IR matching
        ic_true=false;
        [Mu_IR, matchnumL_IR, matchnumR_IR, totalnum_IR] = ...
          ic_2way_liverexchange_3rev(C_L, C_R, Will, Cats, Topo_c456, Prvalues, cplex_true, ic_true);
        
     
        %matching generated by a hypothetical greedy algorithm to show the
        %benefit of optimization; not used in the paper
        [Mu_GR, matchnumL_GR, matchnumR_GR, totalnum_GR] = ...
          greedy_ref_2way_liverexchange_3rev(C_L, C_R, Will, Cats);
        t=toc;
        tic;
        
        %not used in the paper
        max_cyc_size_alt=3;
        weight_true_alt=false;
        NDD=[];
        Same_pat=[];
        
        % Not used in the paper
        Mu_ALT2=zeros(n,1); Matched_true_ALT2=zeros(n,1); totalnum_ALT2=0;
        Mu_ALT3=zeros(n,1); Matched_true_ALT3=zeros(n,1); totalnum_ALT3=0;
        Mu_ALTu=zeros(n,1); Matched_true_ALTu=zeros(n,1); totalnum_ALTu=0;
        
        
        t=toc;
        tic;
        
        %How many transplants without any exchange tehcnology
        matchnumL_NX=sum(Self_L); %Left lobe matches
        matchnumR_NX=sum(Self_R_w); %Right Lobe matches
        totalnum_NX=matchnumR_NX+matchnumL_NX;
        
        
        %calculates doubleequipoise index - not used in the paper
        DE_NX=totalnum_NX/(right_risk*matchnumR_NX+left_risk*matchnumL_NX);
        DE_IR=totalnum_IR/(right_risk*matchnumR_IR+left_risk*matchnumL_IR);
        DE_IC=totalnum_IC/(right_risk*matchnumR_IC+left_risk*matchnumL_IC);
        DE_GR=totalnum_GR/(right_risk*matchnumR_GR+left_risk*matchnumL_GR);
        DE_RSU=totalnum_RSU/(right_risk*matchnumR_RSU+left_risk*matchnumL_RSU);
        
        
        
        Results=[num_sim n w_perc cat456num ...
          matchnumL_NX matchnumR_NX totalnum_NX DE_NX ...
          matchnumL_IC matchnumR_IC totalnum_IC DE_IC ...
          matchnumL_IR matchnumR_IR totalnum_IR DE_IR ...
          matchnumL_GR matchnumR_GR totalnum_GR DE_GR ...
          matchnumL_RSU matchnumR_RSU totalnum_RSU DE_RSU];
        
        %fprintf('\n');
        sR=size(Results,2);
        diary on;
        %fprintf('num_sim    n w_perc Cats456_num NXtotalLn NXtotalRn NXtotalnm 0 ICtotalLn ICtotalRn ICtotalnm 0 IRtotalLn IRtotalRn IRtotalnm 0 GRtotalLn GRtotalRn GRtotalnm \n');
        fprintf('%7i %4i %6.3f %9.3f %9.3f %9.3f %9.3f %9.3f %9.3f %9.3f %9.3f %9.3f %9.3f %9.3f %9.3f %9.3f %9.3f %9.3f %9.3f %9.3f %9.3f %9.3f %9.3f %9.3f \n', Results(1,1:sR));
        %fprintf('\n');
        diary off;
        Res_temp = [Res_temp; Results];
        
      end; %num_sim
     
      
      Stats=[Stats;Res_temp];
      str1t=strcat('Pop',int2str(n));
      
      str2t=strcat('W',int2str(w_perc*100));
      
      
      outfilename=strcat(fnamestr,'ALL-RUNS-',dstr,'-',tstr,'-',str1t,'-',str2t,'.csv');
      outfilename2=strcat(fnamestr,'AVER-STD-',dstr,'-',tstr,'-',str1,'-',str2,'.csv');
      
      csvwrite(outfilename,Stats);
      
      Ansavrstd=[Ansavrstd;mean(Stats);std(Stats)/denomstd];
      fida=fopen(outfilename2,'w');
      asi2=size(Ansavrstd,2);
      asi1=size(Ansavrstd,1);
      
      diary on;
      fprintf(fida, 'num_sim ,   n ,w_perc, Cats456_num ,NXtotalLn ,NXtotalRn ,NXtotalnm ,NXDoubleE ,ICtotalLn, ICtotalRn, ICtotalnm, IRDoubleE, IRtotalLn, IRtotalRn, IRtotalnm ,IRDoubleE, GRtotalLn, GRtotalRn , GRtotalnm, GRDoubleE, RSUtotalLn, RSUtotalRn , RSUtotalnm, RSUDoubleE \n');
      fprintf('num_sim    n w_perc Cats456_num NXtotalLn NXtotalRn NXtotalnm NXDoubleE ICtotalLn ICtotalRn ICtotalnm ICDoubleE IRtotalLn IRtotalRn IRtotalnm IRDoubleE GRtotalLn GRtotalRn GRtotalnm GRDoubleE  RSUtotalL RSUtotalR  RSUtotalm RSUDoublE \n');
      
      %fprintf('%7i %4i %6.3f %9.3f %9.3f %9.3f %9.3f %1i %9.3f %9.3f %9.3f %1i %9.3f %9.3f %9.3f %1i %9.3f %9.3f %9.3f %1i %9.3f %9.3f %9.3f %1i %9.3f %9.3f %9.3f \n', Results(1,1:sR));
      for i=1:asi1
        %fprintf('%7i %4i %6.3f %9.3f %9.3f %9.3f %9.3f %9.3f %9.3f %9.3f %9.3f %9.3f %9.3f %9.3f %9.3f %9.3f %9.3f %9.3f %9.3f %9.3f %9.3f %9.3f %9.3f %9.3f %9.3f %9.3f %9.3f %9.3f %9.3f %9.3f %9.3f %9.3f %9.3f %9.3f %9.3f %9.3f %9.3f  \n');
        fprintf(fida, '%7i, %4i, %6.3f, ',[max_sim Ansavrstd(i,2) Ansavrstd(i,3)]);
        fprintf('%7i %4i %6.3f ',[max_sim Ansavrstd(i,2) Ansavrstd(i,3)]);
        for j=4:asi2
          if mod(i,2)==1
            fprintf(fida, ' %9.3f, ', Ansavrstd(i,j));
            fprintf('%9.3f ', Ansavrstd(i,j));
          else
            fprintf(fida, ' [%6.3f], ', Ansavrstd(i,j));
            fprintf(' [%6.3f] ', Ansavrstd(i,j));
          end;
        end;
        fprintf(fida, ' \n');
        fprintf(' \n');
      end;
      diary off;
      fclose(fida);
    end; %num_size_rate
  end; %num_W_rate
end; %num_pop
save;
save Stats;
save Ansavrstd;
W=[0 0.2 0.4 0.6 0.8 1]';

Xlabs='Willing Fraction';
Ylabs='Transplant Number';

sss50L='K=50  Left-Lobe Transplants';
sss50R='K=50  Right-Lobe Transplants';
sss50T='K=50  Total Transplants';
ymin50=0;
ymax50=45;

sss100L='K=100  Left-Lobe Transplants';
sss100R='K=100  Right-Lobe Transplants';
sss100T='K=100  Total Transplants';
ymin100=0;
ymax100=90;

sss250L='K=250  Left-Lobe Transplants';
sss250R='K=250  Right-Lobe Transplants';
sss250T='K=250  Total Transplants';
ymin250=0;
ymax250=225;

Leg1='No Exchange';
Leg2='Proposed PE&IR&IC';
Leg3='Maximum IR';
Leg4='Ref. Greedy';
Leg5='RSU Priority';

title2='Simulated Transplant Numbers';
LTint=1;

iimin=2;
iimax=2;

maxt=5;
tfs=10;

clf('reset')
for ii=iimin:iimax
  
  AvrL=zeros(6,maxt);
  AvrT=zeros(6,maxt);
  AvrR=zeros(6,maxt);
  SL1=AvrL;
  SL2=AvrL;
  SR1=AvrL;
  SR2=AvrL;
  ST1=AvrL;
  ST2=AvrL;
 
  
  for kk=1:maxt
    col=4+(kk-1)*4+1;
    RowsA=(ii-1)*12+1:2:(ii-1)*12+11;
    RowsS=(ii-1)*12+2:2:(ii-1)*12+12;
    AvrL(:,kk)=Ansavrstd(RowsA,col);
    AvrR(:,kk)=Ansavrstd(RowsA,col+1);
    AvrT(:,kk)=Ansavrstd(RowsA,col+2);
    
    SL1(:,kk)=AvrL(:,kk)-1.96*Ansavrstd(RowsS,col)/sqrt(1000);
    SR1(:,kk)=AvrR(:,kk)-1.96*Ansavrstd(RowsS,col+1)/sqrt(1000);
    ST1(:,kk)=AvrT(:,kk)-1.96*Ansavrstd(RowsS,col+2)/sqrt(1000);
    
    SL2(:,kk)=AvrL(:,kk)+1.96*Ansavrstd(RowsS,col)/sqrt(1000);
    SR2(:,kk)=AvrR(:,kk)+1.96*Ansavrstd(RowsS,col+1)/sqrt(1000);
    ST2(:,kk)=AvrT(:,kk)+1.96*Ansavrstd(RowsS,col+2)/sqrt(1000);
    
  end;
  
  subplot(iimax-iimin+1,3,(ii-iimin)*3+1);
  
  pL=plot(W,AvrL(:,1),'-bo',W,AvrL(:,2),'-r<',W,AvrL(:,3),'-ks',W,AvrL(:,4),'-kx',W,AvrL(:,5),'-m*','MarkerSize',4);
  if ii==1
    title(sss50L,'FontSize',tfs);
    axis([0 1 ymin50 ymax50]);
  elseif ii==2
    title(sss100L,'FontSize',tfs);
    axis([0 1 ymin100 ymax100]);
  else
    title(sss250L,'FontSize',tfs);
    axis([0 1 ymin250 ymax250]);
  end;
  %lgd=legend(Leg1, Leg2, Leg3);
  %lgd.Location='southeast';
  xlabel(Xlabs,'FontSize',8,'FontWeight','bold','Color','k');
  ylabel(Ylabs,'FontSize',8,'FontWeight','bold','Color','k');
  grid on;
  
  subplot(iimax-iimin+1,3,(ii-iimin)*3+2);
  
  pR=plot(W,AvrR(:,1),'-bo',W,AvrR(:,2),'-r<',W,AvrR(:,3),'-ks',W,AvrR(:,4),'-kx',W,AvrR(:,5),'-m*','MarkerSize',4);
  if ii==1
    title(sss50R,'FontSize',tfs);
    axis([0 1 ymin50 ymax50]);
  elseif ii==2
    title(sss100R,'FontSize',tfs);
    axis([0 1 ymin100 ymax100]);
  else
    title(sss250R,'FontSize',tfs);
    axis([0 1 ymin250 ymax250]);
  end;
  %lgd=legend(Leg1, Leg2, Leg3);
  %lgd.Location='southeast';
  xlabel(Xlabs,'FontSize',8,'FontWeight','bold','Color','k');
  ylabel(Ylabs,'FontSize',8,'FontWeight','bold','Color','k');
  grid on;
  
  subplot(iimax-iimin+1,3,(ii-iimin)*3+3);
  
  pT=plot(W,AvrT(:,1),'-bo',W,AvrT(:,2),'-r<',W,AvrT(:,3),'-ks',W,AvrT(:,4),'-kx',W,AvrT(:,5),'-m*','MarkerSize',4);
  if ii==1
    title(sss50T,'FontSize',tfs);
    axis([0 1 ymin50 ymax50]);
  elseif ii==2
    title(sss100T,'FontSize',tfs);
    axis([0 1 ymin100 ymax100]);
  else
    title(sss250T,'FontSize',tfs);
    axis([0 1 ymin250 ymax250]);
  end;
  lgd=legend([pT(3) pT(2) pT(4) pT(5) pT(1)], Leg3, Leg2, Leg4, Leg5, Leg1);
  lgd.Location='southeast';
  lgd.FontSize=6;
  xlabel(Xlabs,'FontSize',8,'FontWeight','bold','Color','k');
  ylabel(Ylabs,'FontSize',8,'FontWeight','bold','Color','k');
  grid on;
end;

savefig(figname2)

clf('reset')
for ii=1:3
  
  AvrL=zeros(6,maxt);
  AvrT=zeros(6,maxt);
  AvrR=zeros(6,maxt);
  SL1=AvrL;
  SL2=AvrL;
  SR1=AvrL;
  SR2=AvrL;
  ST1=AvrL;
  ST2=AvrL;
  
  
  for kk=1:maxt
    col=4+(kk-1)*4+1;
    RowsA=(ii-1)*12+1:2:(ii-1)*12+11;
    RowsS=(ii-1)*12+2:2:(ii-1)*12+12;
    AvrL(:,kk)=Ansavrstd(RowsA,col);
    AvrR(:,kk)=Ansavrstd(RowsA,col+1);
    AvrT(:,kk)=Ansavrstd(RowsA,col+2);
    
    SL1(:,kk)=AvrL(:,kk)-1.96*Ansavrstd(RowsS,col)/sqrt(1000);
    SR1(:,kk)=AvrR(:,kk)-1.96*Ansavrstd(RowsS,col+1)/sqrt(1000);
    ST1(:,kk)=AvrT(:,kk)-1.96*Ansavrstd(RowsS,col+2)/sqrt(1000);
    
    SL2(:,kk)=AvrL(:,kk)+1.96*Ansavrstd(RowsS,col)/sqrt(1000);
    SR2(:,kk)=AvrR(:,kk)+1.96*Ansavrstd(RowsS,col+1)/sqrt(1000);
    ST2(:,kk)=AvrT(:,kk)+1.96*Ansavrstd(RowsS,col+2)/sqrt(1000);
    
  end;
  
  subplot(3,3,(ii-1)*3+1);
  
  pL=plot(W,AvrL(:,1),'-bo',W,AvrL(:,2),'-r<',W,AvrL(:,3),'-ks',W,AvrL(:,5),'-m*','MarkerSize',4);
  if ii==1
    title(sss50L,'FontSize',tfs);
    axis([0 1 ymin50 ymax50]);
  elseif ii==2
    title(sss100L,'FontSize',tfs);
    axis([0 1 ymin100 ymax100]);
  else
    title(sss250L,'FontSize',tfs);
    axis([0 1 ymin250 ymax250]);
  end;
  %lgd=legend(Leg1, Leg2, Leg3);
  %lgd.Location='southeast';
  xlabel(Xlabs,'FontSize',8,'FontWeight','bold','Color','k');
  ylabel(Ylabs,'FontSize',8,'FontWeight','bold','Color','k');
  grid on;
  
  subplot(3,3,(ii-1)*3+2);
  
  
  pR=plot(W,AvrR(:,1),'-bo',W,AvrR(:,2),'-r<',W,AvrR(:,3),'-ks',W,AvrR(:,5),'-m*','MarkerSize',4);
  if ii==1
    title(sss50R,'FontSize',tfs);
    axis([0 1 ymin50 ymax50]);
  elseif ii==2
    title(sss100R,'FontSize',tfs);
    axis([0 1 ymin100 ymax100]);
  else
    title(sss250R,'FontSize',tfs);
    axis([0 1 ymin250 ymax250]);
  end;
  %lgd=legend(Leg1, Leg2, Leg3);
  %lgd.Location='southeast';
  xlabel(Xlabs,'FontSize',8,'FontWeight','bold','Color','k');
  ylabel(Ylabs,'FontSize',8,'FontWeight','bold','Color','k');
  grid on;
  
  subplot(3,3,(ii-1)*3+3);
  
  pT=plot(W,AvrT(:,1),'-bo',W,AvrT(:,2),'-r<',W,AvrT(:,3),'-ks',W,AvrT(:,5),'-m*','MarkerSize',4);
  if ii==1
    title(sss50T,'FontSize',tfs);
    axis([0 1 ymin50 ymax50]);
  elseif ii==2
    title(sss100T,'FontSize',tfs);
    axis([0 1 ymin100 ymax100]);
  else
    title(sss250T,'FontSize',tfs);
    axis([0 1 ymin250 ymax250]);
  end;
  lgd=legend([pT(3) pT(2) pT(4) pT(1)], Leg3, Leg2, Leg5, Leg1);
  lgd.Location='southeast';
  lgd.FontSize=6;
  xlabel(Xlabs,'FontSize',8,'FontWeight','bold','Color','k');
  ylabel(Ylabs,'FontSize',8,'FontWeight','bold','Color','k');
  grid on;
end;

savefig(figname)

diary off;
